
/****************************************************************
  navigation(), this part will calculate the setpoints of the PID that control the servos... 
 ****************************************************************/

void navigation(void)
{
    if ((data_update_event&0x01)==0x01) //Verify if we have new GPS data.. Is like a flag... 
      {
      data_update_event&=0xFE; //Resetting flag, 0xFE is equeal to ~0x01 (not)     
      float dt_t = (float)(millis()-NAV_timer)/1000.0; //Timer
      
    if (AP_Mode == 0x01)   // Check if we are in way-point mode
    {
      if (current_wp < 1) current_wp = 1;
      Set_New_Waypoint(current_wp); //Loading current waypoint.
      update_distance(); //Updating the distance between the current position and the destioantion waypoints
      if (Waypoint_Switcher(wp_distance, wp_radius))    // if Waypoint switcher returns true we've swithed to the next way point.
        update_distance();
    }
    else // If we are in RTL or manual mode -- please return home (waypoint 0)
    { 
       current_wp = 0;
       Set_New_Waypoint(0); // Waypoint zero is home.
       update_distance();
    }
    
    wp_bearing = calc_bearing(lat, lon, wp_current_lat, wp_current_lon); //Calculating bearing
  
    //Now calculates the roll set point, in order to turn to the desired heading.. 
    #if FAKE_BEARING == 1
    roll_set_point= calc_roll(heading_error((int)DESIRED_FAKE_BEARING, ground_course),dt_t); 
    #else
    roll_set_point= calc_roll(heading_error((int)wp_bearing, ground_course),dt_t); //Calculate the roll set point depending the heading error. 
    #endif
    
    NAV_timer=millis(); //Resetting timer    
  }
 
 if((data_update_event&0x02)==0x02)///Checking new GPS data flag
  {
    int alt_error = 0;
    data_update_event&=0xFD; //Clearing flag 
    
    if (AP_Mode == 0x01)   // we are in WP Mode
      alt_error = altitude_error(wp_current_alt, Get_Relative_Altitude()); //Holding defined waypoint altitude
    else
      alt_error = altitude_error(Hold_Current_Altitude(), Get_Relative_Altitude()); //Holding the altitude before switch to RTL. 
    
    //Here will take the altitude error to move control the throttle, has proportional and limits.
    air_speed_hold=constrain(AIRSPEED_CENTRAL+(alt_error*ALTITUDE_ERROR_AIRSPEED_PROPORTIONAL),ALTITUDE_ERROR_AIRSPEED_MIN,ALTITUDE_ERROR_AIRSPEED_MAX); 
    
    //Here will take the altitude error to move the elevator, has proportional and limits
    pitch_set_point=constrain(alt_error*ALTITUDE_ERROR_PITCH_PROPORTIONAL,ALTITUDE_ERROR_PITCH_MIN,ALTITUDE_ERROR_PITCH_MAX);   
    
//    maxis(); //Storing max altitude and speed. 
  }
}

 /****************************************************************
 stabilization(), this part will control the Pitch/Roll of the aircraft.
 ****************************************************************/

void stabilization(void)
{
  unsigned int t_dt = millis() - PID_timer;   // Timer... 
  int pRoll, pPitch;
  
  if (t_dt > PID_dt)    // do this at timed intervals
  {

  // -------

  if (AP_Mode == FLY_BY_WIRE)
    {
       pRoll  = PID_roll( PID_error(roll_trim+ (read_Ch1() / roll_fbw_gain),  get_roll()),  t_dt);
       pPitch = PID_pitch(PID_error(pitch_trim+(read_Ch2() / pitch_fbw_gain), get_pitch()), t_dt);
       pulse_servos(pRoll, pPitch);
       air_speed_hold = fly_by_wire_air_speed_hold;    // FLY_BY_WIRE_SPEED_SETPOINT; 
    }
    
  // -------
  
  else {

    if (gpsFix == GOOD_FIX) // If valid gps data, navigate, else just stabilize... 
      {        
        pRoll = PID_roll(PID_error(roll_trim+roll_set_point, get_roll()), t_dt);
        pPitch = PID_pitch(PID_error(pitch_trim+pitch_set_point+((abs(get_roll())*PITCH_COMP)), get_pitch()), t_dt);
        pulse_servos( pRoll, pPitch );
      }
      else
      {
        if ((abs(read_Ch1())>25) || (abs(read_Ch2())>25))    // Fly by wire, check if we move the sticks 
          {
            pRoll = PID_roll(PID_error(roll_trim+(read_Ch1() / roll_fbw_gain), get_roll()), t_dt);
            pPitch = PID_pitch(PID_error(pitch_trim+(read_Ch2() / pitch_fbw_gain), get_pitch()), t_dt);
            pulse_servos(pRoll, pPitch);
            air_speed_hold = fly_by_wire_air_speed_hold;
          }
       else  // Stabilization only 
       {
        pRoll  = PID_roll( PID_error(roll_trim,  get_roll()), t_dt);
        pPitch = PID_pitch(PID_error(pitch_trim, get_pitch()),t_dt);      
        pulse_servos(pRoll,pPitch);
        air_speed_hold = GPS_ERROR_SPEED_SETPOINT;
       }      
      }

  }

    //-- Throttle
    
    if (AP_Mode == FLY_BY_WIRE)
        pulse_servo_0(fly_by_wire_throttle);
    else {
        throttle_set_point = PID_throttle(air_speed_hold, airSpeed(), t_dt);
        pulse_servo_0(throttle_set_point);
        }
    
    sensor_z();    // read z sensor, update max_ir
    PID_timer = millis();   // resetting timer... 
  } 
}
