// ------------------------------------------------------------------------

void init_ardupilot(void)
{  
  pinMode(2,INPUT);  // Servo input
  pinMode(3,INPUT);  // Servo Input 
  pinMode(4,INPUT);  // MUX pin
  pinMode(5,INPUT);  // Mode pin
  
  digitalWrite(6,HIGH);
  pinMode(6,INPUT);     // Remove Before Fly ground
  pinMode(7,OUTPUT);    // Remove Before Fly input pin
  digitalWrite(7, LOW); // Remove Before Fly Pull Up resistor
  
  pinMode(8,OUTPUT);   // Servo throttle
  pinMode(11,OUTPUT);  // GPS status
  pinMode(12,OUTPUT);  // BLUE LED pin
  pinMode(13,OUTPUT);  // YELLOW LED pin
  
  Init_servos(); 
  pulse_servos(0,0);
  pulse_servo_0(0);

  rx_Ch[0]=pulseIn(2, HIGH, 25000); //Reading the radio sticks central position...
  rx_Ch[1]=pulseIn(3, HIGH, 25000); //Reading the radio sticks central position...
    
  Serial.begin(19200);  // Startup USART
  delay(2000);          // delay for xbee startup...
  
    gpsFix = FLY_BY_WIRE;
    
    Serial.println(" ");
    Serial.println("-- ");
    Serial.println("-- Ardupilot 2.2.3 v 6.6.2");
    Serial.println("-- Setup for Fly by Wire ...");

//    Serial.println("-- ");
//    Serial.println("--> Save Dummy Settings ...");
//    Save_Dummy_Settings();
//    Write_Dummy_Waypoints();

  // ------------------------------------
  
  if(digitalRead(6) == LOW)      // If the "Remove Before Fly" flag is connected... 
  {
   Serial.println("-- Remove before Fly flag is connected...");
    
    // init_gps(); //Initializing GPS
   // Serial.println("-- Waiting for GPS Fix...");
   // Wait_GPS_Fix();//Wait GPS fix...    
   // Serial.println("-- Got GPS, test servos...");
    
    test_servos(); //testing servos
    
//    Save_Dummy_Settings();
    
    Load_Settings();      // Loading saved settings
    Calibrate_IRs();      // Calibrate the IR's (read z sensor)
    
    Serial.println("!!! Setup Jumper is Still Connected !!!");
    Serial.println("!!! Remove before Flying.  !!!");
    delay(1000);
    
    #if FAKE_GPS_LOCK == 0

    while(digitalRead(6) == LOW)    // Loops until we remove the safety flag
    {     
      catch_analogs();
      test_servos(); //Testing servos  
      decode_gps();  //Decoding GPS
      digitalWrite(YELLOW_LED, HIGH);
    }   // Loop till we remove the safety flag.. 

    #else
    Serial.println("Warning GPS in fake mode!!!!!!");
    delay(2000);
    #endif
    
    Save_Launch_Parameters_Flagged();      // Write Launch & Home position to eeprom
  }
 
 // -------  If the "Remove Before Fly" flag is connected. 
 
  else
  { 
    fast_init_gps();
    Load_Settings();
    Restore_Launch_Parameters();
    Calibrate_IRs();
    Serial.println("- ");
    Serial.println("Fast Init.");
  }
  
   // ------------------------------------
   
  digitalWrite(YELLOW_LED, LOW); 
  Serial.println("- ");
  Serial.println("---- Waypoint Data ----");

//  Write_Dummy_Waypoints();
  debug_waypoints();

  current_wp = 0;
  Set_New_Waypoint(0); // Waypoint zero is home.
//  alt_hold = xx + 500;
  
  #if TEST_THROTTLE == 1
  test_throttle();
  #endif
 
}

// ---------------------------------------------------------------------------------

float read_PID_value(byte Adrs)
{
  int tmpWord;
  //float rtnValue;
  
  tmpWord = (int)eeprom_read_word((unsigned int*)Adrs);    // Value stored as 2-Byte Int 
  //rtnValue = tmpWord / 1000.0;    // 495 => 0.495
  //if (tmpWord > 1500) tmpWord = 1500;
  tmpWord = tmpWord & 0x07FF;
  return (float)(tmpWord / 1000.0);
}

// ---------------------------------------------------------------------------------

void Load_Settings(void)
{
  Serial.println(" ");
  Serial.println("-> Load Seettings...");
  
  options = 0;
  //air_speed_offset = 235;
  
  roll_fbw_gain  = int(eeprom_read_byte((byte*)EEPROM_Roll_FBW_Gain));
  pitch_fbw_gain = int(eeprom_read_byte((byte*)EEPROM_Pitch_FBW_Gain));      

  roll_max = eeprom_read_byte((byte*)EEPROM_Roll_Max);
  pitch_max = eeprom_read_byte((byte*)EEPROM_Pitch_Max);
   
  wp_number = eeprom_read_byte((byte*)EEPROM_Num_WP);         // Number of waypoints defined
  wp_radius = eeprom_read_byte((byte*)EEPROM_WP_Radius);      // Radius of the waypoint, normally set to 20 meters.

  fly_by_wire_throttle = eeprom_read_byte((byte*)EEPROM_FBW_Throttle);
  
  air_speed_offset = eeprom_read_word((unsigned int*)EEPROM_AirSp_Bias); //Restoring airspeed bias
  max_alt= (int)eeprom_read_word((unsigned int*)EEPROM_Max_Alt);
  max_spd=(int)eeprom_read_word((unsigned int*)EEPROM_Max_Speed);
  
  kp[kRoll]  = read_PID_value(EEPROM_Roll_Kp);
  kp[kPitch] = read_PID_value(EEPROM_Pitch_Kp);
  ki[kRoll]  = read_PID_value(EEPROM_Roll_Ki);
  ki[kPitch] = read_PID_value(EEPROM_Pitch_Ki);
 
  wordUnion.usWord = eeprom_read_word((unsigned int*)EEPROM_Roll_Trim);
  roll_trim = wordUnion.word;
  wordUnion.usWord = eeprom_read_word((unsigned int*)EEPROM_Pitch_Trim); 
  pitch_trim = wordUnion.word;
  
  Serial.println("-- Settings --");
//  Serial.print("  Options: ");
//  Serial.println((int)options);

  Serial.print("  AirSpd Bias: ");
  Serial.println((int)air_speed_offset);

  Serial.print("  Roll_Trim: ");
  Serial.print(roll_trim);
  Serial.print("  Pitch Trim: ");
  Serial.println(pitch_trim);

  Serial.print("  Roll_Max: ");
  Serial.print((int)roll_max);
  Serial.print("  Pitch Max: ");
  Serial.println((int)pitch_max);

  Serial.print("  Roll FBW Gain: ");
  Serial.print((int)roll_fbw_gain);
  Serial.print("  Pitch FBW Gain: ");
  Serial.println((int)pitch_fbw_gain);

  Serial.print("  max_alt: ");
  Serial.print(max_alt);
  Serial.print("  max_spd: ");
  Serial.println(max_spd);

  Serial.print("  Num WPs: ");
  Serial.println((int)wp_number);
  Serial.print("  Wp Radius: ");
  Serial.println((int)wp_radius);
  Serial.println(" ");

  Serial.print("  Roll  Kp: ");       Serial.print(   kp[kRoll] );
  Serial.print("  -  Roll  Ki: ");    Serial.println( ki[kRoll] );
  
  Serial.print("  Pitch Kp: ");      Serial.print(   kp[kPitch] );
  Serial.print("  -  Pitch Ki: ");   Serial.println( ki[kPitch]);

  Serial.println(" ");
}

// ---------------------------------------------------------------------------------

void Save_Dummy_Settings(void)
{
  options = 0x00;
  roll_trim = 0;
  pitch_trim = -10;
  wp_number = 2;
  wp_radius = 20;
  roll_max = 65;
  pitch_max = 50;
  air_speed_offset = 235;
  fly_by_wire_throttle = 30;
 
  eeprom_busy_wait();
  eeprom_write_byte((byte*)EEPROM_FBW_Throttle, fly_by_wire_throttle);
  
  eeprom_busy_wait();
  eeprom_write_byte((byte*)EEPROM_Roll_Max, roll_max);
  eeprom_busy_wait();
  eeprom_write_byte((byte*)EEPROM_Pitch_Max, pitch_max);
  eeprom_busy_wait();  
  
  eeprom_write_byte((byte*)EEPROM_Num_WP, wp_number);
  eeprom_busy_wait();
  eeprom_write_byte((byte*)EEPROM_WP_Radius, wp_radius);
  eeprom_busy_wait();
  eeprom_write_word((unsigned int*)EEPROM_Max_Alt, 0);       // max alt & airspeed
  eeprom_busy_wait();
  eeprom_write_word((unsigned int*)EEPROM_Max_Speed, 0);
  eeprom_busy_wait();
  eeprom_write_word((unsigned int*)EEPROM_AirSp_Bias, air_speed_offset); 
  eeprom_busy_wait();
  
  eeprom_write_word((unsigned int*)EEPROM_Roll_Kp, 400);      // roll Kp 
  eeprom_busy_wait();
  eeprom_write_word((unsigned int*)EEPROM_Roll_Ki, 100);      // roll Ki 
  eeprom_busy_wait();
  eeprom_write_word((unsigned int*)EEPROM_Pitch_Kp, 1300);    // pitch Kp 
  eeprom_busy_wait();
  eeprom_write_word((unsigned int*)EEPROM_Pitch_Ki, 900);     // pitch Ki 
  eeprom_busy_wait();

  eeprom_write_word((unsigned int*)EEPROM_Roll_Trim, roll_trim);
  eeprom_busy_wait();
  eeprom_write_word((unsigned int*)EEPROM_Pitch_Trim, pitch_trim);
  eeprom_busy_wait();

  eeprom_write_word((unsigned int*)EEPROM_Roll_FBW_Gain, 8);
  eeprom_busy_wait();
  eeprom_write_word((unsigned int*)EEPROM_Pitch_FBW_Gain, 16);
  
  eeprom_busy_wait();
  eeprom_write_word((unsigned int*)EEPROM_Launch_Alt, 0);    //Saving Launch altitude
  eeprom_busy_wait();
  
}

// ---------------------------------------------------------------------------------

void Save_Launch_Parameters_Flagged(void)
{
  launch_alt=alt_MSL;      //Storing launch altitude... 
  air_speed_offset=analog3;
  current_wp=0;

  eeprom_busy_wait(); 
  eeprom_write_byte((byte*)EEPROM_Crnt_WP, current_wp);

//  eeprom_busy_wait(); 
//  eeprom_write_word((unsigned int*)0x01, air_speed_offset);     // Saving air_speed_bias
//  eeprom_write_word((unsigned int*)0x01,analogRead(3));        // Saving Infrared Calibration..

  if (gpsFix == FLY_BY_WIRE) return;

  eeprom_busy_wait();
  eeprom_write_word((unsigned int*)EEPROM_Launch_Alt, launch_alt);    //Saving Launch altitude

  eeprom_busy_wait();
  eeprom_write_dword((unsigned long*)First_WP_Byte, (long)((float)lat*(float)t7));  //Saving home position.

  eeprom_busy_wait();
  eeprom_write_dword((unsigned long*)(First_WP_Byte+4), (long)((float)lon*(float)t7));

  eeprom_busy_wait();
  eeprom_write_word((unsigned int*)(First_WP_Byte+8), launch_alt);    //Saving wp0 altitude

}
/****************************************************************
 ****************************************************************/
void Restore_Launch_Parameters(void)
{
//  eeprom_busy_wait();
//  air_speed_offset=(int)eeprom_read_word((unsigned int*)0x01);//Restoring airspeed bias
  
  #if REMEMBER_LAST_WAYPOINT_MODE == 1
  current_wp = (int)eeprom_read_byte((byte*)EEPROM_Crnt_WP);
  #endif
  
  eeprom_busy_wait();
  launch_alt = (int)eeprom_read_word((unsigned int*)EEPROM_Launch_Alt);   //  Restoring launch altitude from eeprom...  
}
/****************************************************************
 ****************************************************************/
byte Tx_Switch_Status(void) //Return zero when we are in manual mode, return 2 when autopilot mode 0, return 3 when autopilot mode 1...   
{
  #if RADIO_SWITCH_ACTION == 0
  if(digitalRead(4)==HIGH)
  {
    if(digitalRead(5)==HIGH)
      return 0x01; // WP mode 
    else
      return 0x02; // RTL mode
  }
  else
    return 0x00; 
  #endif
  #if RADIO_SWITCH_ACTION == 1
  if(digitalRead(4)==HIGH)
  {
    if(digitalRead(5)==HIGH)
      return 0x02; // RTL mode 
    else
      return 0x01; // wp mode
  }
  else
    return 0x00; 
  #endif
}

/*****************************************************************************
 *****************************************************************************/
float reset(float value)
{
  if(AP_Mode == 0x00)
    return 0; 
  else
    return value; 
}
/*****************************************************************************
 *****************************************************************************/

void maxis(void)
{
 
  if(gpsFix == GOOD_FIX)
  {
    if(alt_MSL>max_alt)
    {
      max_alt=alt_MSL;
      eeprom_busy_wait(); 
      eeprom_write_word((unsigned int*)EEPROM_Max_Alt, max_alt);
    }

    if(ground_speed>max_spd)
    {
      max_spd=ground_speed;
      eeprom_busy_wait(); 
      eeprom_write_word((unsigned int*)EEPROM_Max_Speed, max_spd);
    }
  }
 
}

// -------------------------------------------------------------------------------------

void print_data(void)
{
  static unsigned long timer1=0;
  static byte counter;
  
  if(millis()-timer1 > ATITUDE_RATE_OUTPUT)
  {   
    digitalWrite(YELLOW_LED, HIGH);
    
    AP_Mode = Tx_Switch_Status();    // 0 = Manual; 1 = WP Mode; 2 = Return to Launch mode
    if (AP_Mode > 0) AP_Mode = FLY_BY_WIRE;
    
    if(counter >= POSITION_RATE_OUTPUT)//If to reapeat every second.... 
    {
      Serial.println(" ");
      Serial.print("|-- ArduPilot Mode:");
      Serial.print((int)AP_Mode);

    Serial.print (",rER:");
    Serial.print((int)roll_set_point);
    Serial.print (",Mir:");
    Serial.print(max_ir);
    Serial.print(",");
    Serial.print ("CH1:");
    Serial.print(read_Ch1()/8);
    Serial.print(",");
    Serial.print ("CH2:");
    Serial.print(read_Ch2()/16);
    Serial.print (",PSET:");
    Serial.print(pitch_set_point);
      Serial.println(",***");

  if (gpsFix != FLY_BY_WIRE)
      {  
      Serial.print("|> ");
      Serial.print("LAT:");
      Serial.print((long)((float)lat*(float)t7));
      Serial.print(",LON:");
      Serial.print((long)((float)lon*(float)t7));
      
       //Serial.print(",WLA:");
      //Serial.print((long)((float)wp_current_lat*(float)t7));
      //Serial.print(",WLO:");
      //Serial.print((long)((float)wp_current_lon*(float)t7));
//      Serial.print(" Wp Alt: ");
//      Serial.println(wp_current_alt);
      
      Serial.print (",SPD:");
      Serial.print(ground_speed);    
      Serial.print(",CRT:");
      Serial.print(climb_rate);
      Serial.print (",ALT:");
      Serial.print(Get_Relative_Altitude());
      Serial.print (",ALH:");
      Serial.print(Hold_Current_Altitude());
      Serial.println(",***");
      
//      Serial.print (",CRS:");

      Serial.print ("|< ");
      Serial.print("CRS:");            
      Serial.print(ground_course);
      Serial.print (",BER:");
      Serial.print(wp_bearing);
      Serial.print (",WPN:");
      Serial.print((int)current_wp);
      Serial.print (",DST:");
      Serial.print(wp_distance);
//      Serial.print (",BTV:");
//      Serial.print(Battery_Voltage(analogRead(5)));      
      Serial.print (",ASO:");  // air_speed_offset = analog3;
      Serial.print(air_speed_offset);
      Serial.print (",AN3:");  // air_speed_offset = analog3;
      Serial.print(analog3);
      
      Serial.println(",***");
      }
      
// ---

      Serial.print ("|* ");
      Serial.print("rKp:");            
      Serial.print(kp[kRoll]);
      Serial.print (",rKi:");
      Serial.print(ki[kRoll]);
    
      Serial.print (",pKp:");
      Serial.print(kp[kPitch]);
      Serial.print (",pKi:");
      Serial.print(ki[kPitch]);
      
      Serial.print(",pTm:");
      Serial.print(pitch_trim);     
      Serial.print(",rTm:");
      Serial.print(roll_trim);
      
      Serial.print(",pMx:");
      Serial.print(pitch_max);     
      Serial.print(",rMx:");
      Serial.print(roll_max);
      
      Serial.println(",***");

// ------

    Serial.print("|^ ASP:");
    Serial.print((int)airSpeed());
    Serial.print(",TSP:");
    Serial.print((int)throttle_set_point);

    Serial.print(",THF:");
    Serial.print(fly_by_wire_throttle);
      
    Serial.print (",RLL:");
    Serial.print(get_roll());
    Serial.print (",PCH:");
    Serial.print(get_pitch());    
    Serial.println(",***");

// -----

      counter=0;
      
      //Serial.println(refresh_rate);
      refresh_rate=0;
    }
    else
    {
    counter++;
    }
    /*
    Serial.print("|^ ASP:");
    Serial.print((int)airSpeed());
    Serial.print(",THH:");
    Serial.print((int)throttle_set_point);
    Serial.print (",RLL:");
    Serial.print(get_roll());
    Serial.print (",PCH:");
    Serial.print(get_pitch());    
    Serial.println(",***");
    */
    timer1=millis(); 
    digitalWrite(YELLOW_LED,LOW);
  } 
}
