// ------------------------------------------------------------------------------------------------------------------------------------------------
// XBee.pde - Utility routines for scanning XBee data strings
// ------------------------------------------------------------------------------------------------------------------------------------------------
//
// Be sure to add a call to "check_XBee_data();" in the main ArduPilot loop():
//
/*
void loop()  // Main Loop
{
  navigation();    // Calculate all the setpoints for the roll, pitch and throttle. 
  stabilization(); // Control of all the actuators using IRs and Pressure Sensor...

  catch_analogs();    // Read and average the IR sensors.
  decode_gps();       // Decode GPS data (does nothing during fly-by-wire)
  check_XBee_data();  // Msgs from Gnd Station
  
  print_data();       // Output data to ground station.  Function located in "System" tab.
  
  read_Ch1();         // read the R/C Xmitter Stick positions.
  read_Ch2();
  
  refresh_rate++;
}
// ---------------------------------
*/

#define XBee_STR_SIZE 80

static unsigned char XBeeData[XBee_STR_SIZE];
static unsigned char XBeeStrLen = 0;
uint8_t New_XBee_Data = 0;

// ------------------------------------------------------------------------------------------------------------------------------------------------
/*
long latitude, longitude, distance;
int altitude, holdAltitude, climbRate, groundSpeed;
int course, bearing, waypoint;
int airSpeed, roll, pitch, throttle, AP_Mode;
float battery;
*/
// ------------------------------------------------------------------------------------------------------------------------------------------------

float Search_Float(unsigned char buffer[],char looking[]);
int Search_Int(unsigned char buffer[],char looking[]);
long Search_Long(unsigned char buffer[],char looking[]);

// ------------------------------------------------------------------------------------------------------------------------------------------------
// Called from Main Loop - 
// Interrupts put chars into our ring buffer.
// Process chars - build up XBee strings.

void check_XBee_data(void)
{
  int theData;	
  if (Serial.available()>0) {		// check for waiting UART data
      theData = Serial.read();		// pull 1 chr from ring buffer		
      if (build_up_XBee_string((unsigned char)theData))	// When we have a complete string,
            process_XBee_string();			// process the XBee data.
      }
}

// ------------------------------------------------------------------------------------------------------------------------------------------------
// process incoming chars - commands start with '!' and end with '***'
// return 1 if XBee string is complete - else return zero

unsigned char build_up_XBee_string(unsigned char c)
{
	if (c == '!') {			// this will catch re-starts and stalls as well as valid commands.
		XBeeStrLen = 0;
		XBeeData[XBeeStrLen++] = c;
		return 0;
		}
	
	if (XBeeStrLen != 0)		// string has already started
		{
		if (XBeeStrLen < XBee_STR_SIZE) 
			XBeeData[XBeeStrLen++] = c;
		
		if (c == '*')
			{
			if (XBeeStrLen < XBee_STR_SIZE)
                              XBeeData[XBeeStrLen] = 0;
			if ( (XBeeData[XBeeStrLen-3] == '*') )
				return 1;
			}
		
		//return (c == 0x0D);
		}
		
	return 0;
}

// ------------------------------------------------------------------------------------------------------------------------------------------------
// we have a complete XBee string - extract the data.

void process_XBee_string(void)
{
	uint8_t Unknown_Msg = 1;
	int xairSpeed;
        int tmpWord;
        
	if (XBeeData[6] == 'T') {		// !^ Save Params ***  !-> WrTrm
			Unknown_Msg = 0;
			// save the pitch, roll and air-speed settings for straight and level flight.
                        
                        roll_trim = get_roll();
                        pitch_trim = get_pitch();
                        
                       // eeprom_busy_wait();
                        // eeprom_write_byte((byte*)EEPROM_Roll_Trim, roll_trim);
                       // eeprom_busy_wait();
                       // eeprom_write_byte((byte*)EEPROM_Pitch_Trim, pitch_trim);

                        //Serial.println("|# - Save Msg OK - ***");
			}
		
	else if (XBeeData[6] == 'A') {	          // !^ Save AirSpeed ***  !-> WrAsp:
				Unknown_Msg = 0;
                                fly_by_wire_air_speed_hold = Search_Int(XBeeData,"Asp:");
                                Serial.println("|# - AirSpeed Msg OK - ***");                                
				}

	else if (XBeeData[6] == 'x') {	          // Set Pitch/Roll Min-Max  !-> PMx: 50***
				Unknown_Msg = 0;
                                tmpWord = atoi((char*)(XBeeData+8));
                                if (XBeeData[4] == 'P') {
                                    pitch_max = tmpWord;
                                    eeprom_busy_wait();
                                    eeprom_write_byte((byte*)EEPROM_Pitch_Max, tmpWord);
                                }
                                else {
                                    roll_max = tmpWord;
                                    eeprom_busy_wait();
                                    eeprom_write_byte((byte*)EEPROM_Roll_Max, tmpWord);
                                    }                               
                                Serial.println("|# - Min-Max Msg OK - ***");                                
				}

	else if (XBeeData[6] == 'r') {	          // Set Fly by Wire Throttle  !-> THr: 50***
				Unknown_Msg = 0;
                                tmpWord = atoi((char*)(XBeeData+8));
                                fly_by_wire_throttle = tmpWord;
                                eeprom_busy_wait();
                                eeprom_write_byte((byte*)EEPROM_FBW_Throttle, tmpWord);                               
                                Serial.println("|# - Throttle Msg OK - ***");                                
				}

	else if (XBeeData[6] == 'g') {	          // Set Pitch/Roll FlyByWire Gain  !-> PFg: 16***
				Unknown_Msg = 0;
                                tmpWord = atoi((char*)(XBeeData+8));
                                if (XBeeData[4] == 'P') {
                                    pitch_fbw_gain = tmpWord;
                                    eeprom_busy_wait();
                                    eeprom_write_byte((byte*)EEPROM_Pitch_FBW_Gain, tmpWord);
                                }
                                else {
                                    roll_fbw_gain = tmpWord;
                                    eeprom_busy_wait();
                                    eeprom_write_byte((byte*)EEPROM_Roll_FBW_Gain, tmpWord);
                                    }                                
                                Serial.println("|# - Fly-By_Wire Gain Msg OK - ***");                                
				}

        else if ((XBeeData[6] == 'm') && (XBeeData[4] == 'P'))     // "!-> PTm: 23***"  --  Set Pitch Trim
                              {
				Unknown_Msg = 0;
                                Serial.println("|# - Pitch Trim Msg OK - ***");
                                
                                //tmpWord = Search_Int(XBeeData,"RTm:");
                                //Serial.println(tmpWord);
                                pitch_trim = atoi((char*)(XBeeData+8));
                                //Serial.println(tmpWord);
                                
                                wordUnion.word = pitch_trim;
                                eeprom_write_word((uint16_t*)EEPROM_Pitch_Trim, wordUnion.usWord);
                                
                                Serial.print("pitch_trim: ");
                                Serial.println(pitch_trim);
                              }
          
        else if ((XBeeData[6] == 'm') && (XBeeData[4] == 'R'))     // "!-> RTm: 78"  --  Set Roll Trim
                              {
				Unknown_Msg = 0;
                                //roll_trim = Search_Int(XBeeData,"PTm:");
                                roll_trim = atoi((char*)(XBeeData+8));
                                wordUnion.word = roll_trim;
                                eeprom_write_word((uint16_t*)EEPROM_Roll_Trim, wordUnion.usWord);
                                Serial.println("|# - Roll Trim Msg OK - ***");
                                Serial.println(roll_trim);
                              }
          
  // -----
  	
	else if (XBeeData[5] == 'K') {        // new Kp constant for Roll PID   !-> RKp:
	          Unknown_Msg = 0;
                  if (XBeeData[4] == 'R')    // new PID Roll settings
                        {
                        if (XBeeData[6] == 'p')  
                            {
                            kp[kRoll] = read_new_PID_data( &tmpWord );
                            eeprom_write_word((unsigned int*)EEPROM_Roll_Kp, tmpWord);

                            Serial.println("|# - Kp Msg OK - ***");
                            Serial.println(kp[kRoll]);
                            }
                            
                        else if (XBeeData[6] == 'i')  
                            {
                            ki[kRoll] = read_new_PID_data( &tmpWord );
                            eeprom_write_word((unsigned int*)EEPROM_Roll_Ki, tmpWord);

                            Serial.println("|# - Ki Msg OK - ***");
                            Serial.println(ki[kRoll]);
                            }
                            
                        }
                 
                 else if (XBeeData[4] == 'P')     // new Pitch PID settings
                       {
                         if (XBeeData[6] == 'p')  
                            {
                            kp[kPitch] = read_new_PID_data( &tmpWord );
                            eeprom_write_word((unsigned int*)EEPROM_Pitch_Kp, tmpWord);

                            Serial.println("|# - Kp Msg OK - ***");
                            Serial.println(kp[kPitch]);
                            }
                            
                        else if (XBeeData[6] == 'i')  
                            {
                            ki[kPitch] = read_new_PID_data( &tmpWord );
                            eeprom_write_word((unsigned int*)EEPROM_Pitch_Ki, tmpWord);

                            Serial.println("|# - Ki Msg OK - ***");
                            Serial.println(ki[kPitch]);
                            }
                            
                       }

                  }    // XBeeData = 'K' - pitch & Roll PID settings  


  // -----

	else if (XBeeData[1] == '%') {        // new Kd constant
			Unknown_Msg = 0;
			//newData = atoi(XBeeData+4);
                        Serial.println("|# - Kd Msg OK - ***");
                        }

  // -----
  
	if (Unknown_Msg)
		{
                Serial.println("|~ Unknown Msg ***");
		}
		
	// -----------------------
	
	XBeeStrLen = 0;	// clear len, start building next string
}

// ------------------------------------------------------------------------------------------------------------------------------------------------

// Message is: |-> RKp: 78

float read_new_PID_data( int* tmpWord )
{
  char *BufPointer = (char*)XBeeData;
  float theFloat;
  int cPos = strpos(BufPointer,':');
  *tmpWord = atoi(BufPointer+cPos+1);
  theFloat = (float)(*tmpWord / 1000.0);
  return theFloat;
}

float Search_Float(unsigned char buffer[],char looking[])
{
  char *BufferPointer = (char*)buffer;
  char *SearchString;
  SearchString = strstr (BufferPointer,looking); 
  return atof(SearchString+4);
}

int Search_Int(unsigned char buffer[],char looking[])
{
  char *BufferPointer = (char*)buffer;
  char *SearchString;
  int BufIndex = 4;
  SearchString= strstr (BufferPointer,looking); 
  while (SearchString[BufIndex] == ' ') BufIndex++;
  return atoi(SearchString+BufIndex);
}

long Search_Long(unsigned char buffer[],char looking[])
{
  char *BufferPointer = (char*)buffer;
  char *SearchString;
  SearchString= strstr (BufferPointer,looking); 
  return atol(SearchString+4);
}

// ------------------------------------------------------------------------------------------------------------------------------------------------

unsigned long strTolong(char *s)
{
    short i;
	unsigned long fVal = 0;
    for (i = 0; i < 7; i++)
        fVal = (fVal * 10) + (s[i] - 48);
	return fVal;
}

// ------------------------------------------------------------------------------------------------------------------------------------------------

int strpos( char* s, char c)
{
  int x;
  for (x=0; x< 16; x++) {
    if (s[x] == c) return x;
  }
  return -1;
}

// ------------------------------------------------------------------------------------------------------------------------------------------------

