// ------------------------------------------------------------------------------------------------------------------
// Routines for Button Box
// ------------------------------------------------------------------------------------------------------------------

void buzz(int dur)    // depends on what kind of buzzer is attached
{	
   /*
        digitalWrite(Buzzer, HIGH);
	delay(dur);
	digitalWrite(Buzzer, LOW);
   */
  
    freqout(8700, dur);
}

// -------------------------------------------------------------------------------

void freqout(unsigned long freq, int t)  // freq in hz, t in ms
{
  int hperiod;                               //calculate 1/2 period in us
  long cycles, i;
  pinMode(Buzzer, OUTPUT);                   // turn on output pin

  hperiod = (500000 / freq) - 7;             // subtract 7 us to make up for digitalWrite overhead

  cycles = ((long)freq * (long)t) / 1000;    // calculate cycles

  for (i=0; i<= cycles; i++){              // play note for t ms 
    digitalWrite(Buzzer, HIGH); 
    delayMicroseconds(hperiod);
    digitalWrite(Buzzer, LOW); 
    delayMicroseconds(hperiod - 1);     // - 1 to make up for digitaWrite overhead
  }
pinMode(Buzzer, INPUT);                // shut off pin to avoid noise from other operations

}

// -------------------------------------------------------------------------------
// Blink Heart-Beat LED
// lets me know I'm alive - and we do all periodic tasks here.
// -------------------------------------------------------------------------------

void check_heart_beat(char *state)
{
	unsigned long timerTics = millis();
	unsigned short v;
	
	if ((timerTics < HeartTics) ||
		((timerTics - HeartTics) > 1000)) 	// do this once per second
		{
		HeartTics = millis();
		UpdateFlag = 1;
		
		if (Bad_Data_Ticks > 1)
			{
			//Valid_XBee_Data = 0;
			digitalWrite(Green_LED, LOW);
			digitalWrite(Red_LED, HIGH);
			Bad_Data_Ticks = 3;
			}
		else 
			Bad_Data_Ticks++;
				
		if (*state == 0) {
			digitalWrite(HeartBeat_LED, LOW);
			*state = 1;
			//New_XBee_Data = 1;
			}
		else {
			digitalWrite(HeartBeat_LED, HIGH);
			*state = 0;
			}
			
		} /* do once per second */
	
	
	if ((timerTics < Timer10HzTics) ||
		((timerTics - Timer10HzTics) > 200)) 	// do this 5 times per second
		{
		Timer10HzTics = millis();
		v = read_knob();  // read analog knob and display value on LCD		
		}
			
}

// ------------------------------------------------------------------------------------------------------------------
// Read the rotary knob on the button box.
// return a value based on which menu we're looking at.
// could be a float (0.0 -> 1.0) or int (0->1024 or -512->512) or 0->128

int read_knob(void)
{
	unsigned short v;
	float t;
	char UseStdInt, UseSignedTrim, UseFloat;
	
	v = analogRead(AnalogueKnob);

	if (v < 2) v = 0;
	KnobValue = (KnobValue + v) / 2;
	v = KnobValue;
	
	UseStdInt = 0;
	UseSignedTrim = 0;
	UseFloat = 0;
	
	if (MenuNum > 2) UseStdInt = 1;
	else 
		{ // Pitch & Roll Menus
		if (SubMenuNum == 4) {
			UseStdInt = 1;		// Min/Max values always positive
			v = v / 8;			// range 0..128
			}
		else if (SubMenuNum == 3) UseSignedTrim = 1;	// Trim: +14, -30
		else UseFloat = 1;  // Set Kp, Ki values as floats
		}
	
	LCD_goto(4,0);

	if (UseStdInt) {
			LCD.print(v);
			LCD.print("   ");
			}

	else if (UseSignedTrim) {
			v = (v / 4) - 128;
			LCD.print(v);
			LCD.print("   ");
			}
			
	else if (UseFloat) {	// Kp, Ki values in range 0.0->1.0 
		if (MenuNum == PITCH_MENU) v = v * 2;
		t = (float)(v / 1000.0);
		LCD_put_float(t);
		}
	
	return v;
}

// ------------------------------------------------------------------------------------------------------------------
// Send command and new value to aircraft via XBee radio modem.
// Check which menu / submenu we're looking at to send the proper data.

void send_command( int theMenu, int theSubMenu, int theValue)
{
	switch (theMenu) {
		
		case ROLL_MENU:	// Roll Params
			switch (theSubMenu) {
				case 1: Serial.print("!-> RKp:"); break;	// Set Kp
				case 2: Serial.print("!-> RKi:"); break;	// Set Ki
				case 3: Serial.print("!-> RTm:"); break;	// Set Trim
				case 4: Serial.print("!-> RMx:"); break;	// Set Min.Max
				}
			Serial.print(theValue);
			Serial.println("***");
		break;

		case PITCH_MENU:	// Pitch Params
			switch (theSubMenu) {
				case 1: Serial.print("!-> PKp:"); break;
				case 2: Serial.print("!-> PKi:"); break;
				case 3: Serial.print("!-> PTm:"); break;
				case 4: Serial.print("!-> PMx:"); break;
				}
			Serial.print(theValue);
			Serial.println("***");
		break;

		case THROTTLE_MENU:
			Serial.print("!-> THr:");
			Serial.print(theValue);
			Serial.println("***");
		break;

		case AIRSPEED_MENU:
			Serial.print("!-> ASp:");
			Serial.print(theValue);
			Serial.println("***");
		break;
		
		}
}

// ------------------------------------------------------------------------------------------------------------------
// poll the push buttons to trigger different functions.
// ------------------------------------------------------------------------------------------------------------------

void check_buttons(void)
{
	unsigned short v;
	unsigned long delayTime = millis() - ButtonTime;
	
	if (delayTime < 300) return;

	PortBits = PINB;	// debounce keypress & prevent auto-repeat
	delay(10);
	BtnBits = PINB;
	PortBits = (PortBits & BtnBits) & 0x1F;	      // Buttons are bits 0..4  0b 0001 1111
	BtnBits = BtnBits ^ PrevBits;
	BtnBits = BtnBits & PrevBits;		      // BtnBits now has valid keypresses
	PrevBits = PortBits;


// -------- Check buttons on Ground Station Box

	if (bit_is_set(BtnBits, 0)) {		// top most black button 
		buzz(40);		
		MenuNum++;
		if (MenuNum > LAST_MENU) MenuNum = FIRST_MENU;		
		LCD_clear_screen();
		print_data_to_LCD();
		}

	if (bit_is_set(BtnBits, 1)) {		// middle button
		if (MenuNum < 3) {
			buzz(40);
			SubMenuNum++;
			if (SubMenuNum > 4) SubMenuNum = 1;		
			LCD_clear_screen();
			print_data_to_LCD();
			}
		}

	if (bit_is_set(BtnBits, 2)) {		// bottom button
		buzz(40);
		MenuNum--;
		if (MenuNum < FIRST_MENU) MenuNum = LAST_MENU;		
		LCD_clear_screen();
		print_data_to_LCD();
		}

	// --------
	
	if (bit_is_set(BtnBits, 3)) {	// Red button nearest edge
		v = read_knob();
		LCD_clear_screen();
		if (MenuNum != DSPLY_DATA_MENU)
			{
			buzz(100);
			LCD.print("Sending data...");
			send_command( MenuNum, SubMenuNum, v);
			LCD_clear_screen();
			}
		print_data_to_LCD();
		}

	if (bit_is_set(BtnBits, 4)) {	// Red button closest to knob
		LCD_goto(3,0);
		LCD.print("Save Trim Params");
		//v = read_knob();
		buzz(40);
		Serial.println("!-> WrTrim ***");	// tell airplane to save pitch, roll & airspeed data.		
		LCD_clear_screen();
		}


	ButtonTime = millis();
}

// ------------------------------------------------------------------------------------------------------------------

