/* ArduPilot XBee Ground Staation */
/* Written by Petey the Programmer */
/* 18-Sept-2009 */

#include <ctype.h>
#include <SoftwareSerial.h>

// --- Define IO Pins ---

#define Soft_rxPin 2    // not used
#define LCD_txPin 3     // serial output to LCD
#define Buzzer 4        // simple buzzer output
#define AnalogueKnob 0  // analogue input 

#define TopBtn      8    // push buttons must all be on the same ATMega port
#define MiddleBtn   9    // for the debounce routine to work.  (PORTB pins)
#define BottomBtn   10
#define RedBtn1     11
#define RedBtn2     12

#define Green_LED       5
#define Red_LED         6
#define HeartBeat_LED  13  // Arduino boaard LED

// --- Define Menu numbers ---

#define FIRST_MENU		1
#define LAST_MENU		5

#define ROLL_MENU		1
#define PITCH_MENU		2
#define THROTTLE_MENU	        3
#define AIRSPEED_MENU	        4
#define DSPLY_DATA_MENU         5

// -------------------------------------------------------------------------------
// Global Variables

  char UpdateFlag, New_XBee_Data;
  char BlinkHeartBeatLED, HB_state, Bad_Data_Ticks = 0;;

  int MenuNum, SubMenuNum;
  int XBee_Counter, KnobValue = 0;
  char BtnBits, PortBits, PrevBits = 0;
  unsigned long ButtonTime, HeartTics, Timer10HzTics;
  
  long latitude, longitude, distance;
  int altitude, holdAltitude, climbRate, groundSpeed;
  int course, bearing, waypoint;
  int airSpeed, roll, pitch, throttle, AP_Mode;
  float battery;

  float Roll_Kp, Roll_Ki, Pitch_Kp, Pitch_Ki;
  int Roll_Trim=0, Pitch_Trim=0;
  int Roll_Max=0, Pitch_Max=0;

// -------------------------------------------------------------------------------
// Setup the Serial LCD using the Software serial library.

  SoftwareSerial LCD = SoftwareSerial(Soft_rxPin, LCD_txPin);

// -------------------------------------------------------------------------------

void setup() 
{
  Serial.begin(19200);     // XBee attached to hardware USART on pins 0,1
 
  pinMode(Soft_rxPin, INPUT);    // setup serial LCD
  pinMode(LCD_txPin, OUTPUT);
  LCD.begin(9600);

  // setup push buttons to use pullup resistors
  for (int x=TopBtn; x<=RedBtn2; x++) {
    pinMode(x, INPUT);      
    digitalWrite(x,HIGH);
  }
  
  pinMode(Green_LED, OUTPUT);
  pinMode(Red_LED, OUTPUT);
  pinMode(HeartBeat_LED, OUTPUT);
  
  digitalWrite(Red_LED, HIGH);        // XBee status LEDs
  digitalWrite(Green_LED, LOW);
  
  delay(1000);            // Give LCD and XBee time to start up...
  Serial.println("---");
  Serial.println("XBee Ground Station v1.0");
  
  LCD_clear_screen();      // Clear the LCD display.
  delay(1000);
  LCD.print("Gnd Station v1.0");    // send msg to LCD screen.
  buzz(100);
  
  HeartTics = millis();
  Timer10HzTics = millis();
  ButtonTime = millis();
  
  XBee_Counter = 0;
  UpdateFlag = 0;
  MenuNum = ROLL_MENU;
  SubMenuNum = 1;
} 


// -------------------------------------------------------------------------------

void loop() {

  check_xbee_data();      // read messages from XBee radio modem
  if (New_XBee_Data) {
      print_data_to_LCD();
      New_XBee_Data = 0;
      }

  check_buttons();        // check for user input on button box
  check_heart_beat( &HB_state ); 
  check_xbee_status();    // if valid messages, turn on Green LED
  
}

// -----------------------------------------------------------------------------------------------------------------------------------
