// ------------------------------------------------------------------------------------------------------------------
/* Utility routines for serial LCD */
// ------------------------------------------------------------------------------------------------------------------

void LCD_clear_screen(void)
{
  LCD.print(254, BYTE);      // Clear the LCD display.
  LCD.print(1, BYTE);
}

// ------------------------------------------------------------------------------------------------------------------
// LineNum = [1,2] or [1..4] & [0..19]
// cPos = [0..15]

void LCD_goto(uint8_t lineNum, uint8_t cPos)
{
	uint8_t c = 0;	
	if (lineNum == 2) c = 64;
	if (lineNum == 3) c = 20;
	if (lineNum == 4) c = 84;

	c += (cPos | 0x80);
	
	LCD.print(254, BYTE);
	LCD.print(c, BYTE);
}

// --------------------------------------------------------------------------------------------------------------------------------------------------------------------------

void LCD_put_float(float number)
{
	int tmpWord, divisor;
	char signChr = 0;
	uint8_t index;
	char value[]={0,0,0,0};
	
	if (number < 0) {
		signChr = '-';
		number = -number;
		}
	
	if (number < 1.0) LCD.print('0');
	else if (number < 2.0){
		LCD.print('1');
		number = number - 1.0;
		}
	else if (number < 3.0) {
		LCD.print('2');
		number = number - 2.0;
		}
	else LCD.print('*');
	
	LCD.print('.');
	
	tmpWord = (int)(number * 1000.0);
	divisor = 100;
	for (index = 3; index > 1; index--)
		{
		while((tmpWord - divisor) >= 0)
			{
			tmpWord -= divisor;
			value[index]++;
			}
		value[index] += '0';
		divisor = divisor / 10;
		}		
	value[1] = tmpWord + '0';

	LCD.print(value[3], BYTE);
	LCD.print(value[2], BYTE);
	LCD.print(value[1], BYTE);
}

// ------------------------------------------------------------------------------------------------------------------
// Update the display based on which menu we're looking at.

void print_data_to_LCD(void)
{
	LCD_goto(1,0);			// Goto Home Position on LCD
	
	if ((MenuNum == ROLL_MENU) || (MenuNum == PITCH_MENU))
		{
		if (MenuNum == ROLL_MENU)
			LCD.print("Roll PID Params");
		else
			LCD.print("Pitch PID Params");
			
		LCD_goto(2,0);
		switch (SubMenuNum) {	
			case 1:	LCD.print("Set Kp"); break;
			case 2:	LCD.print("Set Ki"); break;
			case 3:	LCD.print("Set Trim"); break;
			case 4:	LCD.print("Set Min/Max"); break;
			}			
		
		LCD_goto(3,0);
		LCD.print("Crnt Value: ");
		if (MenuNum == ROLL_MENU) {
			switch (SubMenuNum) {	
				case 1:	LCD_put_float(Roll_Kp); break;
				case 2:	LCD_put_float(Roll_Ki); break;
				case 3:	LCD.print(Roll_Trim); break;
				case 4:	LCD.print(Roll_Max); break;
				}
			}	
		else {
			switch (SubMenuNum) {	
				case 1:	LCD_put_float(Pitch_Kp); break;
				case 2:	LCD_put_float(Pitch_Ki); break;
				case 3:	LCD.print(Pitch_Trim); break;
				case 4:	LCD.print(Pitch_Max); break;
				}
			}
		LCD.print("  ");  // erase old data if new value is smaller than old.
		}
	
	else if (MenuNum == THROTTLE_MENU) {
		LCD.print("Throttle Params");
		LCD_goto(3,0);
		LCD.print("Crnt Value: ");
		LCD.print(throttle);
		}
		
	else if (MenuNum == AIRSPEED_MENU) {
		LCD.print("Air Speed Params");
		LCD_goto(3,0);
		LCD.print("Crnt Value: ");
		}
	
	else if (MenuNum == DSPLY_DATA_MENU) {
		LCD.print("In Flight Data");

		LCD_goto(2,0);
		LCD.print("Roll:");
		LCD.print(roll);
		LCD.print(" Pitch:");
		LCD.print(pitch);
		LCD.print(" ");

		LCD_goto(3,0);
		LCD.print("AirSpd:");
		LCD.print(airSpeed);
		}
	
	UpdateFlag = 0;
}

// ------------------------------------------------------------------------------------------------------------------
