// --------------------------------------------------------------------------------------------------------------------------------------------------------------------------
// Utility routines for scanning XBee data strings
// --------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#define XBee_STR_SIZE 80

char XBeeData[XBee_STR_SIZE];
byte XBeeStrLen = 0;


// -----------------------------------------------------------------------------------------------------------------------------------
// update the status LEDs
// If we've received valid data, turn on Green LED
// If no valid data in the last 2 seconds, turn on Red LED

void check_xbee_status(void)
{
		if (XBee_Counter > 125)
		{
			digitalWrite(Red_LED, LOW);
			digitalWrite(Green_LED, HIGH);
							
			XBee_Counter = 0;
			Bad_Data_Ticks = 0;
		}
}

// -----------------------------------------------------------------------------------------------------------------------------
// Called from Main Loop - 
// Process incomming serial chars - build up XBee strings.

void check_xbee_data(void)
{
	char theData;
	
	if (Serial.available()>0) {			// check for waiting xbee data
		theData = Serial.read();		// pull 1 chr from serial buffer
		XBee_Counter++;
		if (build_up_xbee_string(theData))	// When we have a complete string,
			process_xbee_string();			// process the XBee data.
		}
}

// -----------------------------------------------------------------------------------------------------------------------------
// process incoming chars - commands start with '|' and end with '***\r'
// return 1 if XBee string is complete - else return zero

unsigned char build_up_xbee_string(char c)
{
	if (c == '|') {			// this will catch re-starts and stalls as well as valid commands.
		XBeeStrLen = 0;
		XBeeData[XBeeStrLen++] = c;
		return 0;
		}
	
	if (XBeeStrLen != 0)		// string has already started
		{
		if (XBeeStrLen < XBee_STR_SIZE) 
			XBeeData[XBeeStrLen++] = c;
		
		if (c == '*')
			{
			XBeeData[XBeeStrLen] = 0;
			if ( (XBeeData[XBeeStrLen-3] == '*') )
				return 1;
			}
		
		//return (c == 0x0D);
		}
		
	return 0;
}

// -----------------------------------------------------------------------------------------------------------------------------
// we have a complete XBee string - extract the data.

void process_xbee_string(void)
{
	uint8_t Unknown_Msg = 1;
	
	switch (XBeeData[1]) {
	
	case '#':		// |# Msg OK ***
			Unknown_Msg = 0;
			New_XBee_Data = 1;
			
			buzz(100);
			delay(100);
			buzz(100);
			delay(100);
			buzz(100);

			LCD_clear_screen();
			LCD_goto(4,7);
			LCD.print("Data Saved OK");
			break;

	// -----------------------

	case '^':		// |^ ASP:123,THH:18,RLL:6,PCH:-57,***
			Unknown_Msg = 0;
			New_XBee_Data = 1;

			airSpeed = Search_Int(XBeeData,"ASP:"); 
			roll = Search_Int(XBeeData,"RLL:");
			pitch = Search_Int(XBeeData,"PCH:");
			throttle = Search_Int(XBeeData,"THF:");
			break;
			
	// -----------------------
	// |< CRS:0.0,BER:155,WPN:0,DST:11,BTV:0.02,ASO:224,AN3:224.08,***
	
	case '<':
				Unknown_Msg = 0;
				New_XBee_Data = 1;

				course=Search_Int(XBeeData,"CRS:");
				bearing=Search_Int(XBeeData,"BER:");
				waypoint=Search_Int(XBeeData,"WPN:");
				distance=Search_Long(XBeeData,"DST:");
				battery=Search_Float(XBeeData,"BTV:");				
				break;
				
	// |> LAT:-1224566,LON:-12028456,SPD:0.00,CRT:0.00,ALT:7,ALH:7,***

	case '>':
			Unknown_Msg = 0;
			New_XBee_Data = 1;

			latitude = Search_Long(XBeeData,"LAT:"); 
			longitude = Search_Long(XBeeData,"LON:"); 
			altitude = Search_Int(XBeeData,"ALT:");
			holdAltitude = Search_Int(XBeeData,"ALH:");
			climbRate = Search_Int(XBeeData,"CRT:");
			groundSpeed=Search_Int(XBeeData,"SPD:");
			break;
			
	case '*':
			Unknown_Msg = 0;
			New_XBee_Data = 1;

			Roll_Kp = Search_Float(XBeeData,"rKp:"); 
			Roll_Ki = Search_Float(XBeeData,"rKi:"); 
			Roll_Trim = Search_Int(XBeeData,"rTm:");
			Roll_Max  = Search_Int(XBeeData,"rMx:");
			
			Pitch_Kp = Search_Float(XBeeData,"pKp:");
			Pitch_Ki = Search_Float(XBeeData,"pKi:");
			Pitch_Trim = Search_Int(XBeeData,"pTm:");
			Pitch_Max  = Search_Int(XBeeData,"pMx:");
                        break;

	// |-- ArduPilot Mode:1,***
	case '-':
			Unknown_Msg = 0;
			New_XBee_Data = 1;
			AP_Mode = Search_Int(XBeeData,"ode:");
			break;
	
	}
/*
	if (Unknown_Msg)
		{
	//	putstr("Unknown XBee Msg\r\n");
	//	putstr((char*)XBeeData);
	//	putstr("\r\n");

	//		buzz(400);
	//		lcd_goto(4,6);
	//		lcd_put_str(" Unknown XBee Msg  ");
		}
*/		
	// -----------------------
	
	XBeeStrLen = 0;	// clear len, start building next string
}

// -----------------------------------------------------------------------------------------------------------------------------

float Search_Float(char buffer[],char looking[])
{
  char *BufferPointer = (char*)buffer;
  char *SearchString;
  SearchString= strstr (BufferPointer,looking); 
  return atof(SearchString+4);
}

int Search_Int(char buffer[],char looking[])
{
  int BufIndex = 4;
  
  char *BufferPointer = (char*)buffer;
  char *SearchString;
  SearchString= strstr (BufferPointer,looking);
  while (SearchString[BufIndex] == ' ') BufIndex++;
  return atoi(SearchString+BufIndex);
}

long Search_Long(char buffer[],char looking[])
{
  char *BufferPointer = (char*)buffer;
  char *SearchString;
  SearchString= strstr (BufferPointer,looking); 
  return atol(SearchString+4);
}

// -----------------------------------------------------------------------------------------------------------------------------

